# BIAS CORRECTION 

#Guro Heimstad Kliven
#10.03.16, Trondheim

#REMEMBER TO CHANGE THE INPUT AND OUTPUT (buttom of script) FILE NAME

setwd("C:/Users/Bruker/OneDrive - NTNU/Masteroppgave/Temporal downscaling")

require("fExtremes")
require("plyr")
require("reshape")
require("hydroGOF")

#Importing the data

#For the projected climate - CHANGE FOR EACH SCENARIO
projected <- read.csv("FloridaWG_RCP85highstlandetAnnualStats.csv", header = TRUE, sep = ",", fill = TRUE )

#For the bias correction relationship - DON'T CHANGE
dinWG <- read.csv('DailyWG_OBstats.csv', header = TRUE, sep = ",", fill = TRUE )
dAM <- read.table('DailyObservedAM.txt', header = TRUE)

#Observed - find the residual (bias)
####################################

#Include the years in the matirix
Year <- as.numeric(rownames(dinWG))
rownames(dinWG) <- NULL
dWG <- cbind(Year,dinWG)

#Get a matirx with Year and AM
names(dWG)[10] <- "AM"
dWG$AM <- as.numeric(dWG$AM)
dAMWG <- dWG[, c(1, 10)]

#Check the distribution of the daily AM
par(mfrow = c(1, 2))
hist(dAMWG$AM, xlab = 'WG daily AM', breaks = 10)
hist(dAM$AM, xlab = 'Observed daily AM', breaks = 10)

#Make one matrix with observed and WG data
m <- matrix(c(dAM$Year, dAM$AM, dAMWG$AM), nrow = length(dAM$Year))
colnames(m, do.NULL = FALSE)
colnames(m) <- c('Year', 'AMobs', 'AMWG')
m <- as.data.frame(m)

# Rank the precipitation 
AMobs1 <- m$AMobs
AMWG1 <- m$AMWG

AMobsRanked <- sort(AMobs1, decreasing = TRUE)
AMWGRanked <- sort(AMWG1, decreasing = TRUE)

#Calculate plotting position (Cunnane Plotting position)
rank <- c(1:31)
pp <- (rank-0.4)/(31+0.2)
Treturn <- 1/pp

#Make new matrix with plotting position and return values
mpp <- matrix(c(rank, pp, Treturn, AMobsRanked, AMWGRanked), nrow = 31)
colnames(mpp, do.NULL = FALSE)
colnames(mpp) <- c('rank', 'Plotting position', 'Return period', 'AMobs', 'AMWG')
mpp <- as.data.frame(mpp)

AMWG <- mpp$AMWG

#Find the residual: e = diff bwtween observed and WG daily AM
e <- mpp$AMobs - AMWG #the actual point to point difference
res <- predict(lm(e ~ AMWG + I(AMWG^2), data = mpp)) #a regression line of the difference

#Plot the residuals
par(mfrow = c(1,1))
plot(e ~ AMWG, xlab ="SDSM daily AM estimates (mm)", ylab = "Residuals (mm)", 
     cex.axis = 1.5, cex.lab = 1.5)
curve(predict(lm(e ~ AMWG + I(AMWG^2)), newdata = list(AMWG = x)), add = TRUE)
legend("topleft", lty = c(NA , 1), pch = c(1, NA), col = c('black', 'black'), 
       c('Residuals', 'Fitted curve'), cex = 1.5)

#Adjust the observed WG daily AM using the found residual (res): yadj = yWG + e
AMadj <- AMWG + res #Here is the WG daily AM bias corrected

x <- mpp$`Return period`
m <- data.frame(x, mpp$AMobs, mpp$AMWG, AMadj)
xaxis <- c(0, 10, 20, 30, 40, 50, 60)

#Plot observed, WG and WG adjusted against return period
par(mfrow = c(1,1))
with(m, plot(as.numeric(x), m$mpp.AMobs, type = "p", xaxt = "n",
               xlab = "Return period (years)", ylab = "AM daily precipitation (mm)", cex.lab = 1.3, 
             cex.axis =1.2))
lines(x, m$mpp.AMWG, col ='red', lty = 2)
lines(x, m$AMadj)
axis(side = 1, at = xaxis,labels = as.character(xaxis), cex.axis = 1.2)
legend("bottomright", lty = c(NA , 2, 1, NA), pch = c(1, NA, NA, NA), col = c('black', 'red', 'black'), 
       c('Observed', 'SDSM', 'SDSM - bias corrected'), cex = 1.3)


#Goodness of fit tests
######################
#Find RMSE before and after bias correction (mm)
rmseSDSM <- rmse(mpp$AMWG, mpp$AMobs, na.rm=TRUE)
rmaseCORR <- rmse(AMadj, mpp$AMobs, na.rm=TRUE)

#FInd Nash-Sutcliffe coefficient (N-S)
nseSDSM <- NSE(mpp$AMWG, mpp$AMobs, na.rm=TRUE)
nseCORR <- NSE(AMadj, mpp$AMobs, na.rm=TRUE)

#Find the percent bias
pbiasSDSM <- pbias(mpp$AMWG, mpp$AMobs, na.rm=TRUE)
pbiasCORR <- pbias(AMadj, mpp$AMobs, na.rm=TRUE)

#Projections
#############
projected <- sort(projected$Nth.Largest, decreasing = TRUE)
projectedADJ <- projected + res

x <- mpp$`Return period`
m2 <- data.frame(x, mpp$AMobs, projected, projectedADJ)
xaxis <- c(0, 10, 20, 30, 40, 50, 60)

#Plot observed, WG and WG adjusted against return period
par(mfrow = c(1,1))
with(m2, plot(as.numeric(x), m2$mpp.AMobs, type = "p", xaxt = "n",
             xlab = "Return period (years)", ylab = "AM daily precipitation", ylim = c(35, 160)))
lines(x, m2$projected, col ='red', lty = 2)
lines(x, m2$projectedADJ)
axis(side = 1, at = xaxis,labels = as.character(xaxis))
legend("bottomright", lty = c(NA , 2, 1), pch = c(1, NA, NA), col = c('black', 'red', 'black'), 
       c('Observed', 'SDSM projected', 'Adjusted SDSM projected'))

#Bias correction of future climate projections
BiasCorrected <- matrix(c(rank, projectedADJ), nrow = length(Year))
colnames(BiasCorrected, do.NULL = FALSE)
colnames(BiasCorrected) <- c('Year', 'AM')
BiasCorrected

write.table(BiasCorrected, file = "Bias correctedWG_RCP85highstlandet.txt", col.names = TRUE, row.names = FALSE)

