# SCALING FACTORS

#Guro Heimstad Kliven, with supervision from Erle Kristvik
#09.03.17, Trondheim

#NONE OF THE INPUT OR OUTPUT OF THIS FILE SHOULD BE CHANGE AS LONG AS THE 
#OBSERVED DATA IS THE SAME

setwd("C:/Users/Bruker/OneDrive - NTNU/Masteroppgave/Temporal downscaling")

require("evd")

#Import the AM data for different durations
############################################

min2AM <- read.table('min2AM.txt', header = TRUE)
min5AM <- read.table('min5AM.txt', header = TRUE)
min15AM <- read.table('min15AM.txt', header = TRUE)
min30AM <- read.table('min30AM.txt', header = TRUE)

hr1AM <- read.table('hr1AM.txt', header = TRUE)
hr3AM <- read.table('hr3AM.txt', header = TRUE)
hr6AM <- read.table('hr6AM.txt', header = TRUE)
hr12AM <- read.table('hr12AM.txt', header = TRUE)

dAM <- read.table('DailyObservedAM.txt', header = TRUE)

#Estimate the GEV parameters for different durations
#####################################################

#GEV parameters for daily precipitation
##Daily
dGEVfit <- fgev(dAM$AM)                           #Fit the data to the GEV-distribution
dGEVfit
par(mfrow = c(2,2))
plot(dGEVfit)

dpar <- as.numeric(dGEVfit$estimate)              #Get the estimated parameters

dxi <- dpar[3]                                    # xi for daily precipitation
dmu <- dpar[1]                                    # mu for daily precipitation
dbeta <- dpar[2]                                  # beta for daily precipitation

#GEV parameters for the other durations
##2 minutes
min2GEVfit <- fgev(min2AM$AM, shape = dxi)        #Fit data to GEV, with fixed shape parameter
min2GEVfit
par(mfrow = c(2,2))
plot(min2GEVfit)

min2par <- as.numeric(min2GEVfit$estimate)

min2xi <- dxi
min2mu <- min2par[1]
min2beta <- min2par[2]

##5min
min5GEVfit <- fgev(min5AM$AM, shape = dxi)
min5GEVfit
par(mfrow = c(2,2))
plot(min5GEVfit)

min5par <- as.numeric(min5GEVfit$estimate)

min5xi <- dxi
min5mu <- min5par[1]
min5beta <- min5par[2]

##15min
min15GEVfit <- fgev(min15AM$AM, shape = dxi)
min15GEVfit
par(mfrow = c(2,2))
plot(min15GEVfit)

min15par <- as.numeric(min15GEVfit$estimate)

min15xi <- dxi
min15mu <- min15par[1]
min15beta <- min15par[2]

##30min
min30GEVfit <- fgev(min30AM$AM, shape = dxi)
min30GEVfit
par(mfrow = c(2,2))
plot(min30GEVfit)

min30par <- as.numeric(min30GEVfit$estimate)

min30xi <- dxi
min30mu <- min30par[1]
min30beta <- min30par[2]

##1hr
hr1GEVfit <- fgev(hr1AM$AM, shape = dxi)
hr1GEVfit
par(mfrow = c(2,2))
plot(hr1GEVfit)

hr1par <- as.numeric(hr1GEVfit$estimate)

hr1xi <- dxi
hr1mu <- hr1par[1]
hr1beta <- hr1par[2]

##3hr
hr3GEVfit <- fgev(hr3AM$AM, shape = dxi)
hr3GEVfit
par(mfrow = c(2,2))
plot(hr3GEVfit)

hr3par <- as.numeric(hr3GEVfit$estimate)

hr3xi <- dxi
hr3mu <- hr3par[1]
hr3beta <- hr3par[2]

##6hr
hr6GEVfit <- fgev(hr6AM$AM, shape = dxi)
hr6GEVfit
par(mfrow = c(2,2))
plot(hr6GEVfit)

hr6par <- as.numeric(hr6GEVfit$estimate)

hr6xi <- dxi
hr6mu <- hr6par[1]
hr6beta <- hr6par[2]

##12hr
hr12GEVfit <- fgev(hr12AM$AM, shape = dxi)
hr12GEVfit
par(mfrow = c(2,2))
plot(hr12GEVfit)

hr12par <- as.numeric(hr12GEVfit$estimate)

hr12xi <- dxi
hr12mu <- hr12par[1]
hr12beta <- hr12par[2]

#Make a matrix of observed GEV parameters (for later use)
ddistpar <- c(dxi, dmu, dbeta)
min2distpar <- c(min2xi, min2mu, min2beta)
min5distpar <- c(min5xi, min5mu, min5beta)
min15distpar <- c(min15xi, min15mu, min15beta)
min30distpar <- c(min30xi, min30mu, min30beta)
hr1distpar <- c(hr1xi, hr1mu, hr1beta)
hr3distpar <- c(hr3xi, hr3mu, hr3beta)
hr6distpar <- c(hr6xi, hr6mu, hr6beta)
hr12distpar <- c(hr12xi, hr12mu, hr12beta)

mdistpar <- matrix(c(ddistpar, min2distpar, min5distpar, min15distpar, min30distpar, hr1distpar, hr3distpar, hr6distpar, hr12distpar), 
                   nrow = 3)
colnames(mdistpar, do.NULL = FALSE)
colnames(mdistpar) <- c('day', '2min', '5min', '15min', '30min', '60min', '180min', '360min', '720min')
rownames(mdistpar) <- c('xi', 'mu', 'beta')
mdistpar <- as.data.frame(mdistpar)
tmdistpar <- as.data.frame(t(mdistpar))

#Find scaling factors
########################

#Calculate the quantiles as done by Nguyen et al
Treturn <- c(2, 5, 10, 20, 30, 50, 100)  #return period
p <- 1/Treturn                           #probability

##Daily 
dXt <- dmu + (dbeta/dxi)*(1-(1-log(p))^(dxi))
par(mfcol = c(1, 1))
plot(Treturn, dXt, ylab = "Return level: Daily AM (mm)")

##2 min
min2Xt <- min2mu + (min2beta/min2xi)*(1-(1-log(p))^(min2xi))
par(mfcol = c(1, 1))
plot(Treturn, min2Xt, ylab = "Return level (mm)")

##5 min
min5Xt <- min5mu + (min5beta/min5xi)*(1-(1-log(p))^(min5xi))
par(mfcol = c(1, 1))
plot(Treturn, min5Xt, ylab = "Return level (mm)")

##15 min
min15Xt <- min15mu + (min15beta/min15xi)*(1-(1-log(p))^(min15xi))
par(mfcol = c(1, 1))
plot(Treturn, min15Xt, ylab = "Return level (mm)")

##30 min
min30Xt <- min30mu + (min30beta/min30xi)*(1-(1-log(p))^(min30xi))
par(mfcol = c(1, 1))
plot(Treturn, min30Xt, ylab = "Return level (mm)")

##1 hr
hr1Xt <- hr1mu + (hr1beta/hr1xi)*(1-(1-log(p))^(hr1xi))
par(mfcol = c(1, 1))
plot(Treturn, hr1Xt, ylab = "Return level (mm)")

##3 hr
hr3Xt <- hr3mu + (hr3beta/hr3xi)*(1-(1-log(p))^(hr3xi))
par(mfcol = c(1, 1))
plot(Treturn, hr3Xt, ylab = "Return level (mm)")

##6 hr
hr6Xt <- hr6mu + (hr6beta/hr6xi)*(1-(1-log(p))^(hr6xi))
par(mfcol = c(1, 1))
plot(Treturn, hr6Xt, ylab = "Return level (mm)")

##12 hr
hr12Xt <- hr12mu + (hr12beta/hr12xi)*(1-(1-log(p))^(hr12xi))
par(mfcol = c(1, 1))
plot(Treturn, hr12Xt, ylab = "Return level (mm)")

#Find lambda
durationsMinute <- c(2, 5, 15, 30, 60*1, 60*3, 60*6, 12*60)
durationsHour <- durationsMinute/60
lambda <- durationsHour/24 
min2lambda <- lambda[1]
min5lambda <- lambda[2]
min15lambda <- lambda[3]
min30lambda <- lambda[4]
hr1lambda <- lambda[5]
hr3lambda <- lambda[6]
hr6lambda <- lambda[7]
hr12lambda <- lambda[8]

##Calculate the scaling factors based on xi, mu, beta, Xt and lambda
###2 min
min2sxi <- 1
min2smu <- (log(min2mu)-log(dmu))/log(min2lambda)
min2sbeta <- (log(min2beta)-log(dbeta))/log(min2lambda)
min2sXt <- (log(min2Xt)-log(dXt))/log(min2lambda)
###5min
min5sxi <- 1
min5smu <- (log(min5mu)-log(dmu))/log(min5lambda)
min5sbeta <- (log(min5beta)-log(dbeta))/log(min5lambda)
min5sXt <- (log(min5Xt)-log(dXt))/log(min5lambda)
###15 min
min15sxi <- 1
min15smu <- (log(min15mu)-log(dmu))/log(min15lambda)
min15sbeta <- (log(min15beta)-log(dbeta))/log(min15lambda)
min15sXt <- (log(min15Xt)-log(dXt))/log(min15lambda)
###30 min
min30sxi <- 1
min30smu <- (log(min30mu)-log(dmu))/log(min30lambda)
min30sbeta <- (log(min30beta)-log(dbeta))/log(min30lambda)
min30sXt <- (log(min30Xt)-log(dXt))/log(min30lambda)
###1 hr
hr1sxi <- 1
hr1smu <- (log(hr1mu)-log(dmu))/log(hr1lambda)
hr1sbeta <- (log(hr1beta)-log(dbeta))/log(hr1lambda)
hr1sXt <- (log(hr1Xt)-log(dXt))/log(hr1lambda)
###3 hr
hr3sxi <- 1
hr3smu <- (log(hr3mu)-log(dmu))/log(hr3lambda)
hr3sbeta <- (log(hr3beta)-log(dbeta))/log(hr3lambda)
hr3sXt <- (log(hr3Xt)-log(dXt))/log(hr3lambda)
###6 hr
hr6sxi <- 1
hr6smu <- (log(hr6mu)-log(dmu))/log(hr6lambda)
hr6sbeta <- (log(hr6beta)-log(dbeta))/log(hr6lambda)
hr6sXt <- (log(hr6Xt)-log(dXt))/log(hr6lambda)
###12 hr
hr12sxi <- 1
hr12smu <- (log(hr12mu)-log(dmu))/log(hr12lambda)
hr12sbeta <- (log(hr12beta)-log(dbeta))/log(hr12lambda)
hr12sXt <- (log(hr12Xt)-log(dXt))/log(hr12lambda)

#Plot the scaling factors
#########################

#Make a matrix of the quantile scaling factors
qsm <- matrix(c(min2sXt,min5sXt,min15sXt,min30sXt,hr1sXt,hr3sXt,hr6sXt,hr12sXt), 
              nrow = length(min2sXt))
colnames(qsm, do.NULL = FALSE)
colnames(qsm) <- c('2min', '5min', '15min', '30min', '60min', '180min', '360min', '720min')
rownames(qsm) <- c('2', '5', '10', '20', '30', '50', '100')  
qsm <- as.data.frame(qsm)
tqsm <- as.data.frame(t(qsm))

#Make a matrix of the scaling facotrs for the parameters
min2s <- c(min2sxi, min2smu, min2sbeta)
min5s <- c(min5sxi, min5smu, min5sbeta)
min15s <- c(min15sxi, min15smu, min15sbeta)
min30s <- c(min30sxi, min30smu, min30sbeta)
hr1s <- c(hr1sxi, hr1smu, hr1sbeta)
hr3s <- c(hr3sxi, hr3smu, hr3sbeta)
hr6s <- c(hr6sxi, hr6smu, hr6sbeta)
hr12s <- c(hr12sxi, hr12smu, hr12sbeta)

ms <- matrix(c(min2s, min5s, min15s, min30s, hr1s, hr3s, hr6s, hr12s), nrow = length((min2s)))
colnames(ms, do.NULL = FALSE)
colnames(ms) <- c('2min', '5min', '15min', '30min', '60min', '180min', '360min', '720min')
rownames(ms) <- c('xi', 'mu', 'beta')
ms <- as.data.frame(ms)
tms <- as.data.frame(t(ms))

xs <- c(2, 5, 15, 30, 60, 180, 360, 720) #The x axis

#Plot the scaling factors 
par(mfrow = c(1,1))
plot(xs, tms$beta, ylim = c(0, 1), xlab = 'Duration (min)', ylab = 'Scaling factor',
     cex.axis = 1.3, cex.lab = 1.3)
points(xs, tms$mu, col = 'red', pch = 2)
points(xs, tqsm$`2`, pch = 7)
points(xs, tqsm$`5`, pch = 1)
points(xs, tqsm$`10`, col = 'red', pch = 23)
points(xs, tqsm$`20`, pch = 23)
points(xs, tqsm$`30`, col = 'red', pch = 18)
points(xs, tqsm$`50`,  pch = 18)
points(xs, tqsm$`100`, col = 'red', pch = 4)
legend("topleft", title = 'Scaling factors', pch = c(1, 2, 7, 1, 23, 23, 18, 18, 4), 
       col = c('black', 'red', 'black','black', 'red', 'black', 'red', 'black', 'red'),
       c('beta', 'mu', '2 years return', '5 years return', '10 years return', '20 years return', 
         '30 years return', '50 years return', '100 years return'), cex = 1, ncol = 2)

##Find the average of the derived scaling factors (excluding 12 hr duration 
####because this point is divirging from the rest)
sfactors1 <- c(min2smu, min2sbeta, min5smu, min5sbeta, min15smu, min15sbeta, min30smu, min30sbeta, 
                hr1smu, hr1sbeta, hr3smu, hr3sbeta, hr6smu, hr6sbeta, tqsm$`2`[1:7], tqsm$`5`[1:7], 
               tqsm$`10`[1:7], tqsm$`20`[1:7], tqsm$`30`[1:7], tqsm$`50`[1:7], tqsm$`100`[1:7])

sfactor <- mean(sfactors1)

xs2 <- xs[1:7]

#Plot the scaling factors and the average. 12 hr duration is excluded
par(mfrow = c(1,1))
plot(xs2, tms$beta[1:7], ylim = c(0.2, 1), xlab = 'Duration (min)', ylab = 'Scaling factor',
     cex.axis = 1.3, cex.lab = 1.3)
points(xs2, tms$mu[1:7], col = 'red', pch = 2)
points(xs2, tqsm$`2`[1:7], pch = 7)
points(xs2, tqsm$`5`[1:7], pch = 1)
points(xs2, tqsm$`10`[1:7], col = 'red', pch = 23)
points(xs2, tqsm$`20`[1:7], pch = 23)
points(xs2, tqsm$`30`[1:7], col = 'red', pch = 18)
points(xs2, tqsm$`50`[1:7],  pch = 18)
points(xs2, tqsm$`100`[1:7], col = 'red', pch = 4)
abline(a = sfactor, b = 0, lty = 3)
legend("topright", title = 'Scaling factors', pch = c(1, 2, 7, 1, 23, 23, 18, 18, 4, NA), 
       lty = c(NA, NA, NA, NA, NA, NA, NA, NA, NA, 3), 
       col = c('black', 'red', 'black','black', 'red', 'black', 'red', 'black', 'red', 'black'),
       c('beta', 'mu', '2 years return', '5 years return', '10 years return', '20 years return', 
         '30 years return', '50 years return', '100 years return', 'Average scaling factor'), 
       cex = 1, ncol= 2)


#Export the GEV parameters and scaling factor for further use
##############################################################
write.table(tmdistpar, file = "Observed GEV parameters.txt", row.names = TRUE, col.names = TRUE)
write.table(sfactor, file = "Scaling factor.txt")
