# IDF curve for observed precipitation

#Guro Heimstad Kliven
#Trondheim, 09.03.17

#REMEMBER TO CHANGE OUTPUT NAME IF YOU ADD CLIMATE FACTORS TO THE IDF CURVE
#THE INPUT FILE IS ALLWAYS THE SAME
#THE CLIMATE FACTOR IS CHANGED IN LINE 111

setwd("C:/Users/Bruker/OneDrive - NTNU/Masteroppgave/Temporal downscaling")

#Import the observed GEV parameters
###################################

tmdistpar <- read.table("Observed GEV parameters.txt")

dxi <- tmdistpar[1,1]
dmu <- tmdistpar[1,2]
dbeta <- tmdistpar[1,3]

min2xi <- tmdistpar[2,1]
min2mu <- tmdistpar[2,2]
min2beta <- tmdistpar[2,3]

min5xi <- tmdistpar[3,1]
min5mu <- tmdistpar[3,2]
min5beta <- tmdistpar[3,3]

min15xi <- tmdistpar[4,1]
min15mu <- tmdistpar[4,2]
min15beta <- tmdistpar[4,3]

min30xi <- tmdistpar[5,1]
min30mu <- tmdistpar[5,2]
min30beta <- tmdistpar[5,3]

hr1xi <- tmdistpar[6,1]
hr1mu <- tmdistpar[6,2]
hr1beta <- tmdistpar[6,3]

hr3xi <- tmdistpar[7,1]
hr3mu <- tmdistpar[7,2]
hr3beta <- tmdistpar[7,3]

hr6xi <- tmdistpar[8,1]
hr6mu <- tmdistpar[8,2]
hr6beta <- tmdistpar[8,3]

hr12xi <- tmdistpar[9,1]
hr12mu <- tmdistpar[9,2]
hr12beta <- tmdistpar[9,3]

#Estimate the quantiles for IDF cunstruction
############################################
Treturn <- c(2, 5, 10, 20, 30, 50, 100)       #return period
p <- 1/Treturn                                #probability

##Daily 
dXt <- dmu - (dbeta/dxi)*(1-(-log(1-p))^(-dxi)) #From Coles 2001
par(mfcol = c(1, 1))
plot(Treturn, dXt, ylab = "Return level: Daily AM (mm)")

##2 min
min2Xt <- min2mu - (min2beta/min2xi)*(1-(-log(1-p))^(-min2xi))
par(mfcol = c(1, 1))
plot(Treturn, min2Xt, ylab = "Return level (mm)")

##5 min
min5Xt <- min5mu - (min5beta/min5xi)*(1-(-log(1-p))^(-min5xi))
par(mfcol = c(1, 1))
plot(Treturn, min5Xt, ylab = "Return level (mm)")

##15 min
min15Xt <- min15mu - (min15beta/min15xi)*(1-(-log(1-p))^(-min15xi))
par(mfcol = c(1, 1))
plot(Treturn, min15Xt, ylab = "Return level (mm)")

##30 min
min30Xt <- min30mu - (min30beta/min30xi)*(1-(-log(1-p))^(-min30xi))
par(mfcol = c(1, 1))
plot(Treturn, min30Xt, ylab = "Return level (mm)")

##1 hr
hr1Xt <- hr1mu - (hr1beta/hr1xi)*(1-(-log(1-p))^(-hr1xi))
par(mfcol = c(1, 1))
plot(Treturn, hr1Xt, ylab = "Return level (mm)")

##3 hr
hr3Xt <- hr3mu - (hr3beta/hr3xi)*(1-(-log(1-p))^(-hr3xi))
par(mfcol = c(1, 1))
plot(Treturn, hr3Xt, ylab = "Return level (mm)")

##6 hr
hr6Xt <- hr6mu - (hr6beta/hr6xi)*(1-(-log(1-p))^(-hr6xi))
par(mfcol = c(1, 1))
plot(Treturn, hr6Xt, ylab = "Return level (mm)")

##12 hr
hr12Xt <- hr12mu - (hr12beta/hr12xi)*(1-(-log(1-p))^(-hr12xi))
par(mfcol = c(1, 1))
plot(Treturn, hr12Xt, ylab = "Return level (mm)")

##Make a new matrix with all durations and return values 
m <- matrix(c(min2Xt, min5Xt, min15Xt, min30Xt, hr1Xt, hr3Xt, hr6Xt, hr12Xt, dXt), 
            nrow = length(Treturn))
colnames(m, do.NULL = FALSE)
colnames(m) <- c("2min", "5min", "15min","30 min", "60min",
                 "180 min", "360 min", "720 min", "1440min")
rownames(m) <- c("2years", "5years", "10years", "20years", "30years", 
                 "50years", "100years")
m <- m*1.2  #Adding climate factor

tm <- as.data.frame(t(m)) # Transform the matrix
xIDF <- c(2, 5, 15, 30, 60, 180, 360, 720, 1440)
tmIDF <- (tm/xIDF)*60 #From depth (mm) to intensity (mm/hr)
x <- factor(c(2, 5, 15, 30, 60, 180, 360, 720, 1440)) #Values on x-axis
dat <- data.frame(x, tm)
datIDF <- data.frame(x, tmIDF)

##Plot the DDF curve
with(dat, plot(x, dat$X100years, type = "l", xaxt = 'n')) #To get uneven distance on x axis
with(dat, plot(as.numeric(x), tm$`100years`, type = "l", xaxt = "n", 
               xlab = 'Duration (min)', ylab = 'Depth (mm)')) # points instead of steps
lines(x, dat$X50years, col = "red", lty = 2)
lines(x, dat$X30years, col = "blue", lty = 3)
lines(x, dat$X20years, lty = 6)
lines(x, dat$X10years, col = "51",  lty = 5)
lines(x, dat$X5years, col = "red", lty = 4)
lines(x, dat$X2years, col = "blue")
axis(side = 1, at = x,labels = as.character(dat$x)) #Get exactly the value I want on x-axis
legend("topleft", title = 'Return period (years)', lty = c(1, 2, 3, 6, 5, 4, 1), 
       col = c('black', 'red', 'blue','black', "51", 'red', 'blue'),
       c('100', '50', '30', '20', '10', '5', '2'))

#Plot the IDF curve
with(datIDF, plot(x, datIDF$X100years, type = "l", xaxt = 'n')) #To get uneven distance on x axis
with(datIDF, plot(as.numeric(x), tmIDF$`100years`, type = "l", xaxt = "n", 
                  xlab = 'Duration (min)', ylab = 'Intensity (mm/hr)')) # points instead of steps
lines(x, datIDF$X50years, col = "red", lty = 2)
lines(x, datIDF$X30years, col = "blue", lty = 3)
lines(x, datIDF$X20years, lty = 6)
lines(x, datIDF$X10years, col = "51",  lty = 5)
lines(x, datIDF$X5years, col = "red", lty = 4)
lines(x, datIDF$X2years, col = "blue")
axis(side = 1, at = x,labels = as.character(datIDF$x)) #Get exactly the value I want on x-axis
legend("topright", title = 'Return period (years)', lty = c(1, 2, 3, 6, 5, 4, 1), 
       col = c('black', 'red', 'blue','black', "51", 'red', 'blue'),
       c('100', '50', '30', '20', '10', '5', '2'))

write.table(tmIDF, "IDFobserved_KF20.txt", col.names = TRUE, row.names = TRUE)
