# DIVIDING MINUTE DATA INTO INTERVALS

##Guro Heimstad Kliven, with supervision from Erle Kristvik
#Trondheim, 15.02.17

#DON'T CHANGE THE INPUT AS LONG AS THE OBSERVED DATA IS THE SAME

setwd("C:/Users/Bruker/OneDrive - NTNU/Masteroppgave/Temporal downscaling")

require("fExtremes")
require("esd")

#Import the data
################

minute <- read.table("MinuteObserved.txt", header = TRUE)
minute <- minute[complete.cases(minute),]

#Divide the data into different durations
#########################################

#Hours
pr <- as.numeric(as.character(minute$PRCP))
pr[pr==999] <- NA
n <- nrow(minute)

year <- as.character(minute$Year)
month <- as.character(minute$Month)
day <- as.character(minute$Day)
time <- as.character(minute$Time)

for(i in 1:n) {
  if (as.numeric(month[i]) < 10) {
    month[i] <- paste("0", month[i], sep = "")
  }
}
for(i in 1:n) {
  if (as.numeric(day[i]) < 10) {
    day[i] <- paste("0", day[i], sep = "")
  }
}


t <- c(1:n)
t <- as.POSIXlt(paste(paste(year, month, day, sep = "-"), time, sep = " "), format = "%Y-%m-%d %H:%M")


hr1 <- aggregate(list(PRCP = pr), 
                 list(datetime = cut(t, "1 hour")), 
                 sum)

hr3 <- aggregate(list(PRCP = pr), 
                 list(datetime = cut(t, "3 hour")), 
                 sum)

hr6 <- aggregate(list(PRCP = pr), 
                 list(datetime = cut(t, "6 hour")), 
                 sum)

hr12 <- aggregate(list(PRCP = pr), 
                  list(datetime = cut(t, "12 hour")), 
                  sum)

hr24 <- aggregate(list(PRCP = pr), 
                  list(datetime = cut(t, "24 hour")), 
                  sum)

#Minutes
min2 <- aggregate(list(PRCP = pr),
                  list(datetime = cut(t, "2 min")), 
                  sum)    

min5 <- aggregate(list(PRCP = pr),
                 list(datetime = cut(t, "5 min")), 
                 sum)                               #Summarized from current minute to next 5 minute,
                                                    #e.g.: prcp at 18:00 contains of the prec from
                                                    #18:00 to and including 18:04
min10 <- aggregate(list(PRCP = pr), 
                  list(datetime = cut(t, "10 min")), 
                  sum)                              #The same system as for minute, e.g. 18:00 - 18:09

min15 <- aggregate(list(PRCP = pr), 
                   list(datetime = cut(t, "15 min")), 
                   sum) 

min20 <- aggregate(list(PRCP = pr), 
                   list(datetime = cut(t, "20 min")), 
                   sum) 

min30 <- aggregate(list(PRCP = pr), 
                   list(datetime = cut(t, "30 min")), 
                   sum) 
#Remove all missing values
hr1 <- hr1[complete.cases(hr1),]
hr3 <- hr3[complete.cases(hr3),]
hr6 <- hr6[complete.cases(hr6),]
hr12 <- hr12[complete.cases(hr12),]
hr24 <- hr24[complete.cases(hr24),]
hr48 <- hr48[complete.cases(hr48),]

min1 <- min1[complete.cases(min1),]
min2 <- min2[complete.cases(min2),]
min5 <- min5[complete.cases(min5),]
min10 <- min10[complete.cases(min10),]
min15 <- min15[complete.cases(min15),]
min20 <- min20[complete.cases(min20),]
min30 <- min30[complete.cases(min30),]

#Find the annual maximum value (AM) for the different durations
###############################################################

##Minutes
min2AM <- aggregate(list(AM = min2$PRCP),
                    list(datetime = cut(as.Date(min2$datetime), "1 year")),
                    max, na.action("na.pass"))

min5AM <- aggregate(list(AM = min5$PRCP),
                    list(datetime = cut(as.Date(min5$datetime), "1 year")),
                    max)

min10AM <- aggregate(list(AM = min10$PRCP),
                    list(datetime = cut(as.Date(min10$datetime), "1 year")),
                    max)

min15AM <- aggregate(list(AM = min15$PRCP),
                     list(datetime = cut(as.Date(min15$datetime), "1 year")),
                     max)

min20AM <- aggregate(list(AM = min20$PRCP),
                     list(datetime = cut(as.Date(min20$datetime), "1 year")),
                     max)

min30AM <- aggregate(list(AM = min30$PRCP),
                    list(datetime = cut(as.Date(min30$datetime), "1 year")),
                    max)

##Hours
hr1AM <- aggregate(list(AM = hr1$PRCP),
                     list(datetime = cut(as.Date(hr1$datetime), "1 year")),
                     max)

hr3AM <- aggregate(list(AM = hr3$PRCP),
                   list(datetime = cut(as.Date(hr3$datetime), "1 year")),
                   max)

hr6AM <- aggregate(list(AM = hr6$PRCP),
                   list(datetime = cut(as.Date(hr6$datetime), "1 year")),
                   max)

hr12AM <- aggregate(list(AM = hr12$PRCP),
                   list(datetime = cut(as.Date(hr12$datetime), "1 year")),
                   max)

hr24AM <- aggregate(list(AM = hr24$PRCP),
                   list(datetime = cut(as.Date(hr24$datetime), "1 year")),
                   max)

hr48AM <- aggregate(list(AM = hr48$PRCP),
                   list(datetime = cut(as.Date(hr48$datetime), "1 year")),
                   max)

#Save the AM for different durations
####################################
write.table(min2AM, "min2AM.txt", col.names = TRUE, row.names = FALSE)
write.table(min5AM, "min5AM.txt", col.names = TRUE, row.names = FALSE)
write.table(min10AM, "min10AM.txt", col.names = TRUE, row.names = FALSE)
write.table(min15AM, "min15AM.txt", col.names = TRUE, row.names = FALSE)
write.table(min20AM, "min20AM.txt", col.names = TRUE, row.names = FALSE)
write.table(min30AM, "min30AM.txt", col.names = TRUE, row.names = FALSE)

write.table(hr1AM, "hr1AM.txt", col.names = TRUE, row.names = FALSE)
write.table(hr3AM, "hr3AM.txt", col.names = TRUE, row.names = FALSE)
write.table(hr6AM, "hr6AM.txt", col.names = TRUE, row.names = FALSE)
write.table(hr12AM, "hr12AM.txt", col.names = TRUE, row.names = FALSE)
write.table(hr24AM, "hr24AM.txt", col.names = TRUE, row.names = FALSE)


