#Prepare Norway Florida climate data. Starts 1983 through 2015
#Manually removed titles and summary statistics from the txt file in notepad. 
#After these lines are removed, the code should work. 

setwd("C:/Users/Bruker/OneDrive - NTNU/Masteroppgave/SDSM Florida")
require(plyr)
require(reshape)
raw <- read.table('Florida2.txt', header = T, sep = ";", fill = T)
raw <- raw[-c(32,64,65,97,129,130,162,194,226,227,259,291,323,355,356,388,389,421,422,454,455,487,488,520,521,553,554,586,587,619,620,652,653,685,686,718,719,751,752,784,785,817,818,850,851,883,884,916,917,949,950,982,983,1015,1016,1048,1049,1081,1082),]
raw$Jan <- gsub("-1", "NA", raw$Jan)
raw$Feb <- gsub("-1", "NA", raw$Feb)
raw$Mar <- gsub("-1", "NA", raw$Mar)
raw$Apr <- gsub("-1", "NA", raw$Apr)
raw$May <- gsub("-1", "NA", raw$May)
raw$Jun <- gsub("-1", "NA", raw$Jun)
raw$Jul <- gsub("-1", "NA", raw$Jul)
raw$Aug <- gsub("-1", "NA", raw$Aug)
raw$Sep <- gsub("-1", "NA", raw$Sep)
raw$Oct <- gsub("-1", "NA", raw$Oct)
raw$Nov <- gsub("-1", "NA", raw$Nov)
raw$Dec <- gsub("-1", "NA", raw$Dec)

raw$Jan <- gsub("-999", "NA", raw$Jan)
raw$Feb <- gsub("-999", "NA", raw$Feb)
raw$Mar <- gsub("-999", "NA", raw$Mar)
raw$Apr <- gsub("-999", "NA", raw$Apr)
raw$May <- gsub("-999", "NA", raw$May)
raw$Jun <- gsub("-999", "NA", raw$Jun)
raw$Jul <- gsub("-999", "NA", raw$Jul)
raw$Aug <- gsub("-999", "NA", raw$Aug)
raw$Sep <- gsub("-999", "NA", raw$Sep)
raw$Oct <- gsub("-999", "NA", raw$Oct)
raw$Nov <- gsub("-999", "NA", raw$Nov)
raw$Dec <- gsub("-999", "NA", raw$Dec)

names(raw) <- c("Day", "01", "02", "03", "04", "05", "06", "07", "08", "09", "10", "11", "12")
raw$year <- rep(1983:2016, each=31)

mdata <- melt(raw, id=c("Day","year"))
names(mdata)[3] <- "Month"
names(mdata)[4] <- "PRCP"
mdata$NewDate <- paste0(mdata$Month,"-",mdata$Day,"-", mdata$year)

mdata$PRCP<-as.character(mdata$PRCP)
mdata$PRCP<-as.numeric(mdata$PRCP)
mdata$NewDate <- strptime(mdata$NewDate, tz = "EST", format = "%m-%d-%Y")
mdata <- arrange(mdata, NewDate)

#Shifting the precipitation up one step to fit to SDSM
#shift <- function(x, n) `length<-`(tail(x, -n), length(x)) 
#mdataS <- transform(mdata, PRCP = shift(PRCP, 1)) 

# Remove precipitation data before 1985 (to get 30 yrs data)
treshold <- '1985-01-01' # must be adjusted for each input file
mdata1 <- subset(mdata, mdata[ , 5] > treshold)

# Remove precipitation data after 31.12.2015
mdata2 <- subset(mdata1, mdata1$NewDate <= '2016-01-01')

# The following step if for checking whether the data is exponentially distributed
# (see script ExpCheck)
write.table(mdata2,"DailyPRCP.txt")

#count how many missing dates you have to see how bad (or good) the data may be. Should be no more than 
# 30 % missing
#Values listed as TRUE here (meaning they are "na" values) are the missing rows of data
count(is.na(mdata2$PRCP)==TRUE)

# plot daily precipitation
plot(mdata2$PRCP,
     type = "l",
     xlab = "Day",
     ylab = "Daily Precipitation")


#SDSM requires the start date of the data to be Jan 1 1948.
#So we will create a file of dates starting here and merge it with our current file
days48_2010 <- seq(as.Date("1948-01-01"), as.Date("2015-12-31"), "days")
days48_2010<-as.data.frame(days48_2010)
names(days48_2010)[1] <- "NewDate"
mdata2$NewDate <- as.character(mdata$NewDate)
days48_2010$NewDate <- as.character(days48_2010$NewDate)
mdata3 <- merge(days48_2010, mdata2, by.x = 'NewDate', by.y = 'NewDate', all.x = T)

#Replace blanks with -999 in all data. This is expected format for SDSM
mdata3 <- replace(mdata3, is.na(mdata3), -999)


#Remove extra columns we made for summary purposes
floridaPRCP <- subset(mdata3, select=-c(1, 2, 3, 4))

write.table(floridaPRCP[,1],"floridaPRCP.txt", 
            row.names = FALSE, col.names = FALSE)


